README for Lan-Tunes media server
=================================

Dependencies
------------
The following packages are required for lan-tunes:
* pygtk

The following packages are optional depending on the configuration:
* libvlc
* pyinotify
* pywebkitgtk
* xset
* ffprobe
* festival

In order to use the vlc plugin /src/backends/vlc/*.py must be able to 
include the python vlc bindings. There are two solutions to this, either 
have vlc.py in your standard python installation or download vlc.py from 
[The VLC git repository](http://git.videolan.org/?p=vlc/bindings/python.git;a=blob;f=generated/vlc.py) 
and put it in /src/backends/vlc/.

Notes
-----

THE SERVER RUNS ON PORT 8080 and iptables redirects 80 to 8080.

Suggested iptables command:
> iptables -t nat -A PREROUTING -p tcp --dport 80 -j REDIRECT --to-port 8080

I use proftpd and let users upload to /var/music and have this as an 
inotify'd directory so that users can easily upload over FTP.

Copyright Notice
----------------

This project is licensed under the MIT license. Copyright (c) 2012 Allan 
Wirth, et all.

On making new frontends and backends
------------------------------------

Creating new plugins for Lan-Tunes is easy. Plugins fall into two 
categories, frontends and backends. Backends plugins add new media types to 
display, whereas frontend plugins mostly either modify the playlist state 
or respond to changes in the playlist. For example, youtube is a backend 
plugin whereas last.fm is a frontend.

### Frontend

Creating a new frontend plugin is easy. A frontend plugin is merely a 
module (or package, if the plugin requires multiple source files) that 
defines a class Plugin that subclasses FrontendPlugin (from 
src/frontendplugin.py). Plugin must have the same constructor signature as 
FrontendPlugin and must also call FrontendPlugin's constructor. Frontend 
plugins are designed to run in their own thread, although they are not 
required to (for example, the lastfm plugin has no thread of its own).

Frontend plugins that require their own thread should override the 
functions run() and kill(). run() will be called in a separate thread, and 
calling kill from the main thread SHOULD cause run to return (and ideally 
join with it). Note: Because run will be in a separate thread, all calls to 
GUI methods must be made through the util.idle_add interface in order to 
avoid locking problems.

Frontend plugins can also connect to the methods of the playlistmanager 
object that they are passed in the constructor.

For an example of a simple, non-threaded plugin see 
src/frontends/lastfm/__init__.py.

For additional documentation see src/frontendplugin.py.

### Backend

Backend plugins are slightly more complicated. Like a frontend plugin, a 
backend plugin is merely a module (or package) that defines a class Plugin 
that subclasses BackendPlugin (from src/backendplugin.py). Again, Plugin 
must have the same constructor signature as BackendPlugin and must call 
BackendPlugin's constructor. Unlike frontend plugins, backend plugins do 
not explicitly have their own threads (although they can for playback).

The following methods of BackendPlugin must be overridden:

* __init__(self, backends, conf) - Should passthrough to 
backendplugin.__init__. backends contains a dictionary of the names of the 
already loaded modules mapped to their Plugin.
* get_widget(self) - Should return a gtk widget that will be used for url 
that have 'video': True in their metadata. Will be called in the gtk main 
loop. This method is REQUIRED unless get_metadata never sets the video 
field to true.
* can_play(self, url) - Returns true if this backend can play a given url. 
This MIGHT be called OUTSIDE of the gtk main loop. This method is REQUIRED.
* play(self, url) - Plays a url. This may be called outside the gtk main 
loop. Sometime after a call to play, the plugin MUST emit("stopped") in the 
gtk main loop (potentially using util.idle_add). This method is REQUIRED. 
This function MUST NOT BLOCK.
* get_metadata(self, url) - Retrieves metadata for a url. This will be 
called OUTSIDE the main loop. This should return a dictionary containing at 
least the key 'video' which will determine if the the gui will use the 
widget while playing this url. It can (and should) also contain the keys 
'title', and 'duration'. Additionally 'artist' and 'album' are optional, 
but recommended if available. This function is expected to block for some 
indeterminate amount of time (ideally short). This method is REQUIRED.
* set_volume(self, val) - This sets the volume of the backend. This will be 
called OUTSIDE of the main loop. val will be an integer between 0 (min 
volume) and 100 (max volume). This method is REQUIRED.

Backend plugins MUST also define a variable DEPS that contains an iterable 
of names of plugins that this plugin depends upon.

For a simple example of a backend plugin see 
/src/backends/youtube/__init__.py. For an example of a plugin that wraps 
the core VLC plugin see /src/backends/xhamster/__init__.py.

For additional documentation see /src/backendplugin.py.

### Notes

Read the documentation in frontendplugin.py and backendplugin.py
in order to understand the methods that must be implemented. A new
plugin will be a package or module that has in its root a class called
Plugin which inherients from frontendplugin or backendplugin (based
on which kind of plugin it is) that should have the same constructor
and method signatures. For plugins that require multiple files, a 
package is required instead of a module in order to create some
organization.

Backends MUST also have a DEPS global variable that is a sequence
object that contains strings naming other backend plugins that must be
loaded before the described plugin (and subsequently passed to its
constructor as the first argument). This is a system for having backends
that extend other backends by depending on them. The effects of ciruclar
dependencies are undefined.
