# Copyright (c) 2012 Allan Wirth <allan@allanwirth.com>
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation
# files (the "Software"), to deal in the Software without
# restriction, including without limitation the rights to use,
# copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following
# conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

"""Base class from which all backends must derive."""

import gobject

class BackendPlugin(gobject.GObject):
  """Base class for backends."""
  __gsignals__ = {
    "stopped": (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ())
  }
  def __init__(self, backends, conf):
    """Config is a ConfigParser.RawConfigParser that is the
    configuration object for the runtime. backends is a dict
    of names mapping to other plugins that have been initialized."""
    gobject.GObject.__init__(self)

    self.config = conf
  def get_widget(self):
    """Should return a gtk widget that will be used for urls
    that have 'video': True in their metadata."""
    pass
  def can_play(self, uri):
    """Returns true if this backend can play a given uri. This
    is called OUTSIDE of the gtk main loop."""
    pass
  def play(self, uri):
    """Plays a uri. This may be called outside the gtk main loop."""
    pass
  def get_metadata(self, uri):
    """Retrieves metadata for a uri. This will be called outside the
    main loop. This should return a dictionary containing at least
    the key 'video' which will determine if the the gui will use
    the widget while playing this uri. It can (and should) also
    contain the keys 'title', and 'duration'. 'artist' and 'album'
    are optional, but recommended if available. This function is
    expected to block."""
    pass
  def set_volume(self, val):
    """This sets the volume of the backend. This will be called outside
    of the main loop. val will be an integer between 0 (min volume) and
    100 (max volume)."""
    pass
