# Copyright (c) 2012 George Silvis, III <george.iii.silvis@gmail.com>
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation
# files (the "Software"), to deal in the Software without
# restriction, including without limitation the rights to use,
# copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following
# conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

"""Soundcloud backend."""

from ... import backendplugin

from . import soundcloud

DEPS = ("vlc",)

class Plugin(backendplugin.BackendPlugin):
  def __init__(self, plugins, config):
    backendplugin.BackendPlugin.__init__(self, plugins, config)
    self.vlc = plugins["vlc"]
    self.vlc.connect("stopped", self._stopped_event)
  def _stopped_event(self, event):
    self.emit("stopped")
  def get_widget(self):
    return self.vlc.get_widget()
  def can_play(self, uri):
    return "soundcloud.com" in uri and (
             uri.startswith("http://") or uri.startswith("https://"))
  def play(self, uri):
    self.vlc.play(soundcloud.get_soundcloud_track_url(uri))
  def stop(self):
    self.vlc.stop()
  def get_metadata(self, uri):
    result = soundcloud.get_soundcloud_metadata(uri)
    return result
  def set_volume(self, val):
    self.vlc.set_volume(val)
