# Copyright (c) 2012 Allan Wirth <allan@allanwirth.com>
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation
# files (the "Software"), to deal in the Software without
# restriction, including without limitation the rights to use,
# copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following
# conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

import os
import threading

from ... import backendplugin
from ... import util
from . import texttospeech

DEPS = ("vlc",)

class Plugin(backendplugin.BackendPlugin):
  def __init__(self, plugins, config):
    backendplugin.BackendPlugin.__init__(self, plugins, config)
    self.vlc = plugins["vlc"]
    self.vlc.connect("stopped", self._stopped_event)
    self.tmp = None
  def _cleanup(self, url):
    try:
      os.unlink(url)
      os.rmdir(os.path.split(url)[0])
    except Exception as e:
      pass
  def _stopped_event(self, event):
    if self.tmp:
      self._cleanup(self.tmp)
      self.tmp = None
    self.emit("stopped")
  def get_widget(self):
    return None
  def can_play(self, url):
    return texttospeech.isText(url)
  def play(self, uri):
    self.tmp = util.get_temp_filename(self.config, "text.wav")
    def wrapper():
      tmp = self.tmp
      texttospeech.getWav(uri,tmp)
      if self.tmp == tmp: # otherwise got dequeued while rendering
        self.vlc.play("file://"+self.tmp)
      else:
        self._cleanup(tmp)
    threading.Thread(target=wrapper).run()
  def stop(self):
    self.vlc.stop()
  def get_metadata(self, uri):
    return texttospeech.getMetadata(uri)
  def set_volume(self, val):
    self.vlc.set_volume(val)
