# Copyright (c) 2012 Allan Wirth <allan@allanwirth.com>
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation
# files (the "Software"), to deal in the Software without
# restriction, including without limitation the rights to use,
# copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following
# conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

"""Vimeo backend."""

from ... import backendplugin
from ... import util

from . import vimeo

DEPS = ()

class Plugin(backendplugin.BackendPlugin):
  def __init__(self, plugins, conf):
    backendplugin.BackendPlugin.__init__(self, plugins, conf)

    self.widget = vimeo.VimeoWidget()
    self.widget.connect("stopped", self._stopped_event)
  def _stopped_event(self, event):
    self.emit("stopped")
  def get_widget(self):
    return self.widget
  def can_play(self, url):
    return vimeo.get_vimeo_id(url) != ""
  def play(self, uri):
    util.idle_add(self.widget.load_id, vimeo.get_vimeo_id(uri))
  def stop(self):
    util.idle_add(self.widget.unload)
    util.idle_add(self.emit, "stopped")
  def get_metadata(self, uri):
    result = vimeo.get_metadata(vimeo.get_vimeo_id(uri))
    return result
  def set_volume(self, val):
    util.idle_add(self.widget.set_volume, val)
