# Copyright (c) 2012 Allan Wirth <allan@allanwirth.com>
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation
# files (the "Software"), to deal in the Software without
# restriction, including without limitation the rights to use,
# copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following
# conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

"""Functions for handling vimeo interaction."""

import urlparse
import urllib2
import os
import webkit
import gobject
import gtk
import json

from ... import util

VIMEO_API_URL = "http://vimeo.com/api/v2/video/{0}.json"

VIMEO_PAGE = open(util.get_resource_path("vimeo_page.html"), "r").read()

VIMEO_SET_VOL = """
try {{
  document.getElementById("player").contentWindow.postMessage(JSON.stringify({{"method":"setVolume", "value": {0}}}), "*");
}} catch (e) {{ }};
"""

def get_vimeo_id(url):
  if not url.startswith("http://") and not url.startswith("https://"):
    url = "http://" + url
  try:
    match = urlparse.urlparse(url)
    if "vimeo.com" in match.netloc and match.path[1:].isdigit():
      return match.path[1:]
    else:
      return ""
  except Exception as e:
    return ""

def get_metadata(id):
  result = {"video": True}

  try :
    api = urllib2.urlopen(VIMEO_API_URL.format(id))
    obj = json.load(api)
    result["title"] = obj[0]["title"];
    result["duration"] = obj[0]["duration"] * 1000;
  except Exception as e:
    print e
    pass
  return result

class VimeoWidget(webkit.WebView):
  __gsignals__ = {
    "stopped" : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
  }
  def __init__(self):
    webkit.WebView.__init__(self)
    self.connect("title-changed", self._callback_wrapper)
    self.load_uri("about:blank")

    self.volume = 0
    self.first_title = True
  def _callback_wrapper(self, widget, frame, title):
    # I think this gets called out of loop? WTF
    util.idle_add(self._actual_callback, widget, frame, title)
  def _actual_callback(self, widget, frame, title):
    if "Stopped" in title:
      self.unload()
      self.emit("stopped")
    elif "Inited" in title:
      self._force_reshow()
      self._change_volume()
    elif "Playing" in title or "Paused" in title:
      self._change_volume()
  def _force_reshow(self, *args):
    # hacky solution to force realloc and draw of flash
    if self.get_property("visible"):
      rect = self.get_allocation()
      rect2 = gtk.gdk.Rectangle(rect.x, rect.y, rect.width-1, rect.height-1)
      self.size_allocate(rect2)
      self.size_allocate(rect)
  def unload(self):
    if self.can_go_back():
      self.go_back()
    else:
      self.load_uri("about:blank")
  def load_id(self, id):
    self.unload()
    self.load_string(VIMEO_PAGE.format(id), "text/html", "UTF-8", "http://player.vimeo.com/")
  def set_volume(self, volume):
    self.volume = volume
    self._change_volume()
  def get_volume(self):
    return self.volume
  def _change_volume(self):
    self.execute_script(VIMEO_SET_VOL.format(self.volume/100.))
