# Copyright (c) 2012 Allan Wirth <allan@allanwirth.com>
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation
# files (the "Software"), to deal in the Software without
# restriction, including without limitation the rights to use,
# copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following
# conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

"""VLC backend module."""

import threading
import time
import os
import urllib

from ... import backendplugin
from ... import util

from . import vlc
from . import vlcwidget
from . import ffprobe

DEPS = ()

class Plugin(backendplugin.BackendPlugin):
  def __init__(self, plugins, config):
    backendplugin.BackendPlugin.__init__(self, plugins, config)

    self.instance = vlc.Instance("-q", "--no-video-title-show")
    self.player = self.instance.media_player_new()
    self.widget = vlcwidget.VLCWidget(self.player)
    # NOTE: MUST HOLD EVENT_MANAGER REFERENCE OR ELSE VLC WILL GC IT
    self.event_manager = self.player.event_manager()
    self.event_manager.event_attach(vlc.EventType.MediaPlayerEndReached,
                                    self._stopped_event)
    self.event_manager.event_attach(vlc.EventType.MediaPlayerEncounteredError,
                                    self._stopped_event)
  def _stopped_event(self, event):
    util.idle_add(self.emit, "stopped")
  def get_widget(self):
    return self.widget
  def can_play(self, uri):
    return True
  def play(self, uri):
    self.player.set_media(self.instance.media_new(uri))
    self.player.play()
  def stop(self):
    self.player.pause()
    util.idle_add(self.emit, "stopped")
  def get_metadata(self, uri):
    media = self.instance.media_new(uri)
    media.parse()
    result = {}
    result["title"] = media.get_meta(vlc.Meta.Title)
    result["artist"] = media.get_meta(vlc.Meta.Artist)
    result["album"] = media.get_meta(vlc.Meta.Album)
    for k, v in result.iteritems():
      if v:
        result[k] = v.decode("UTF-8")
    if (uri.startswith("file://") and
        os.path.isfile(urllib.unquote(uri[len("file://"):]))):
      ffprobe_meta = ffprobe.get_media_info(urllib.unquote(uri[len("file://"):]))
      if ffprobe_meta["duration"] is not None:
        result["duration"] = ffprobe_meta["duration"]
      result["video"] = ffprobe_meta["video"]
    else:
      result["duration"] = media.get_duration()
      result["duration"] = result["duration"] if result["duration"] else None
      result["video"] = False
    return result
  def set_volume(self, val):
    self.player.audio_set_volume(val)
