# Copyright (c) 2012 Allan Wirth <allan@allanwirth.com>
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation
# files (the "Software"), to deal in the Software without
# restriction, including without limitation the rights to use,
# copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following
# conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

"""ffprobe interface."""

import subprocess
import re

def get_media_info(filename):
  result = subprocess.Popen(["ffprobe", filename],
    stdout = subprocess.PIPE, stderr = subprocess.STDOUT)
  meta = {"video": False, "duration": None}
  for line in result.stdout:
    line = line.strip()

    match = re.search("Duration: ([0-9]+):([0-9]+):([0-9]+).([0-9]+),", line)
    if match is not None:
      dur =  int(match.group(1)) * (60 * 60 * 1000)
      dur += int(match.group(2)) * (60 * 1000)
      dur += int(match.group(3)) * 1000
      dur += int(match.group(4)) * 10
      meta["duration"] = dur
      continue
    match = re.search("Stream #[0-9]+.[0-9]+: ([a-zA-Z]+?):", line)
    if match is not None:
      meta["video"] = meta["video"] or match.group(1) == "Video"
      continue
  return meta

if __name__ == "__main__":
  import sys
  print get_media_info(sys.argv[1])
