# Copyright (c) 2012 Allan Wirth <allan@allanwirth.com>
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation
# files (the "Software"), to deal in the Software without
# restriction, including without limitation the rights to use,
# copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following
# conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

"""Functions for handling for youtube interaction."""

import urlparse
import urllib2
import os
import webkit
import gobject
import gtk
import xml.dom.minidom

from ... import util

YOUTUBE_URL = "http://www.youtube.com/watch_popup?v={0}&vq={1}#t=0"
YOUTUBE_API_URL = "http://gdata.youtube.com/feeds/api/videos/{0}?v=2"

YOUTUBE_SET_VOL = """
try {{
  yt.player.playerReferences_.player1.api.setVolume({0});
}} catch (e) {{ }};
"""

YOUTUBE_INJECT = open(util.get_resource_path("youtube_inject.js"),
                      "r").read()

def get_youtube_id(url):
  if not url.startswith("http://") and not url.startswith("https://"):
    url = "http://" + url
  try:
    match = urlparse.urlparse(url)
    if "youtu.be" in match.netloc:
      return match.path[1:]
    elif not "youtube" in match.netloc:
      return ""
    return urlparse.parse_qs(match.query)["v"][0]
  except Exception as e:
    print e
    return ""

def get_metadata(id):
  result = {"video": True}

  try :
    api = urllib2.urlopen(YOUTUBE_API_URL.format(id))
    obj = xml.dom.minidom.parse(api)
    result["title"] = obj.getElementsByTagName("media:title")[0].firstChild.data;
    if " - " in result["title"] :
      tmp = result["title"].partition(" - ")
      result["title"] = tmp[2]
      result["artist"] = tmp[0]
    result["duration"] = int(obj.getElementsByTagName("yt:duration")[0].getAttribute("seconds")) * 1000
  except:
    pass
  return result

class YoutubeWidget(webkit.WebView):
  __gsignals__ = {
    "stopped" : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
  }
  def __init__(self):
    webkit.WebView.__init__(self)
    self.connect("title-changed", self._callback_wrapper)
    self.load_uri("about:blank")

    self.volume = 0
  def _callback_wrapper(self, widget, frame, title):
    # I think this gets called out of loop? WTF
    util.idle_add(self._actual_callback, widget, frame, title)
  def _actual_callback(self, widget, frame, title):
    if "YouTube" in title:
      self.execute_script(YOUTUBE_INJECT)
    elif "Stopped" in title:
      self.unload()
      self.emit("stopped")
    elif "Inited" in title:
      self._force_reshow()
      self._change_volume()
    elif "Playing" in title or "Paused" in title:
      self._change_volume()
  def _force_reshow(self, *args):
    # hacky solution to force realloc and draw of flash
    if self.get_property("visible"):
      rect = self.get_allocation()
      rect2 = gtk.gdk.Rectangle(rect.x, rect.y, rect.width-1, rect.height-1)
      self.size_allocate(rect2)
      self.size_allocate(rect)
  def unload(self):
    if self.can_go_back():
      self.go_back()
    else:
      self.load_uri("about:blank")
  def load_id(self, id, quality):
    url = YOUTUBE_URL.format(id, quality)
    self.unload()
    self.load_uri(url)
  def set_volume(self, volume):
    self.volume = volume
    self._change_volume()
  def get_volume(self):
    return self.volume
  def _change_volume(self):
    self.execute_script(YOUTUBE_SET_VOL.format(self.volume))
