# Copyright (c) 2012 Allan Wirth <allan@allanwirth.com>
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation
# files (the "Software"), to deal in the Software without
# restriction, including without limitation the rights to use,
# copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following
# conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

"""Plays youtube videos immediately from JSON requests."""

import json
import socket
import time

from ... import util
from ... import frontendplugin

BUF_SIZE = 4096
TIMEOUT = .1

class Plugin(frontendplugin.FrontendPlugin):
  def __init__(self, player, conf):
    frontendplugin.FrontendPlugin.__init__(self, player, conf)

    self.halted = False
    self.addr = self.config.get("door", "addr")
    self.port = self.config.getint("door", "port")
  def run(self):
    config = self.config
    player = self.player

    conn = None
    while not self.halted:
      if conn is None:
        try:
          conn = socket.socket()
          conn.connect((self.addr, self.port))
          conn.settimeout(TIMEOUT)
        except socket.error:
          conn = None
          time.sleep(TIMEOUT)
      else:
        try:
          data = conn.recv(BUF_SIZE)
          obj = json.loads(data)
        except socket.timeout:
          continue
        except socket.error as e:
          try:
            conn.close()
          except socket.error:
            pass
          conn = None
          continue
        except:
          continue
        if "youtube" in obj and obj["youtube"] and obj["youtube"] != "BLANK":
          self.player.add_item(obj["youtube"], False, True)
    if conn is not None:
      try:
        conn.close()
      except socket.error:
        pass

  def kill(self):
    self.halted = True
    self.join()
