# Copyright (c) 2012 Allan Wirth <allan@allanwirth.com>
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation
# files (the "Software"), to deal in the Software without
# restriction, including without limitation the rights to use,
# copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following
# conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

"""HTTP frontend for remote control."""

import threading
import BaseHTTPServer
import cgi
import SocketServer
import urlparse
import urllib

from ... import util
from ... import config
from ... import frontendplugin

_HEADER = open(util.get_resource_path("header.inc.html"),
               "r").read().replace("{NAME}", config.app_name)

_TABLE_ROW = open(util.get_resource_path("table_row.inc.html"),
                  "r").read()

_QUEUE_REMOVE = open(util.get_resource_path("queue_remove.inc.html"),
                     "r").read()

_FOOTER = open(util.get_resource_path("footer.inc.html"),
               "r").read().replace("{NAME}",
               config.app_name).replace("{LINK}", config.app_url)

class _RequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
  protocol_version = "HTTP/1.1"
  def do_GET(self):
    parsed = urlparse.urlparse(self.path)
    path = parsed.path
    if path == "/":
      page = _HEADER + self.get_list()
      page += _FOOTER.replace("{VOL}",
                      str(self.server.thread.player.get_current_volume()))
      self.do_page(page)
    elif path == "/queue":
      self.do_queue()
    else:
      self.do_not_found()
  def do_POST(self):
    parsed = urlparse.urlparse(self.path)
    path = parsed.path
    environ = {"REQUEST_METHOD": "POST",
               "QUERY_STRING": "",
               "CONTENT_LENGTH": int(self.headers["Content-Length"]),
               "CONTENT_TYPE": cgi.parse_header(self.headers["Content-Type"])[0]}
    if path == "/changevol":
      data = cgi.parse(self.rfile, environ=environ)
      try:
        self.server.thread.player.modify_current_volume(int(data["val"][0]))
      except KeyError:
        pass
      self.do_page(str(self.server.thread.player.get_current_volume()))
    elif path == "/queue":
      data = cgi.parse(self.rfile, environ=environ)
      try:
        if data["change"][0] == "move":
          amount = int(data["amount"][0])
          moved = int(data["moved"][0])
          up = True if amount > 0 else False
          if not up:
            amount *= -1
          for i in range(amount):
            if up:
              self.server.thread.player.move_up(moved)
            else:
              self.server.thread.player.move_down(moved)
        elif data["change"][0] == "remove":
          removed = int(data["removed"][0])
          self.server.thread.player.remove(removed)
      except KeyError:
        pass
      self.do_queue()
    elif path == "/submit":
      data = cgi.parse(self.rfile, environ=environ)
      try:
        self.server.thread.player.add_item(data["uri"][0], False)
      except KeyError:
        pass
      self.do_page("Success")
    elif path == "/upload":
      if (not self.headers.has_key("Content-Length") or
          int(self.headers["Content-Length"]) >
          self.server.thread.config.getint("http", "max_filesize")):
        self.do_page("Failure - Content length too large", 200)
        self.rfile.read()
        return
      query = cgi.FieldStorage(fp=self.rfile,
                               headers=self.headers,
                               environ=environ)
      upfile = query["file"]
      tmp_f = util.get_temp_filename(self.server.thread.config,
                                     upfile.filename)
      open(tmp_f, "wb").write(upfile.value)
      self.server.thread.player.add_item("file://"+urllib.quote(tmp_f), True)

      self.do_page("Success")
    else:
      self.do_not_found()
  def do_not_found(self):
    self.do_page("404 Not Found", 404)
  def do_queue(self):
    self.do_page(self.get_list())
  def do_page(self, content, response=200, type="text/html"):
    self.send_response(response)
    self.send_header("Content-type", type+"; charset=UTF-8")
    self.send_header("Content-Length", str(len(content)))
    self.send_header("Connection", "close")
    self.end_headers()
    self.wfile.write(content)
  def get_list(self):
    queue = self.server.thread.player.get_queue()
    result = ""
    total_time = 0
    for i in range(len(queue)):
      row = queue[i]
      title = (row.metadata["title"] if "title" in row.metadata
                 and row.metadata["title"] else "N/A")
      artist = (row.metadata["artist"] if "artist" in row.metadata
                 and row.metadata["artist"] else "N/A")
      album = (row.metadata["album"] if "album" in row.metadata
                 and row.metadata["album"] else "N/A")
      time = (util.format_time(row.metadata["duration"]) if
                "duration" in row.metadata and row.metadata["duration"]
                else "N/A")
      total_time += (row.metadata["duration"]
                     if "duration" in row.metadata and row.metadata["duration"]
                     else 0)
      even_odd = "odd" if i%2 else "even"
      result += _TABLE_ROW.format(title, artist, album, time,
                                  row.id, row.uri, even_odd, _QUEUE_REMOVE)
    result += _TABLE_ROW.format("Total:", "", "",
                                util.format_time(total_time), "", "", "", "")
    return result
  def log_request(self, code=None, size=None):
    pass
  def log_error(self, *args):
    pass
  def log_message(self, format, *args):
    pass

class _HTTPServer(SocketServer.ThreadingMixIn, BaseHTTPServer.HTTPServer):
  daemon_threads = True
  def __init__(self, addr, port):
    BaseHTTPServer.HTTPServer.__init__(self, (addr, port), _RequestHandler)
  def handle_error(self, request, client_address):
    pass

class Plugin(frontendplugin.FrontendPlugin):
  def __init__(self, player, conf):
    frontendplugin.FrontendPlugin.__init__(self, player, conf)

    self.addr = conf.get("http", "bind_addr")
    self.port = conf.getint("http", "bind_port")
    self.server = None
  def run(self):
    self.server = _HTTPServer(self.addr, self.port)
    self.server.thread = self
    self.server.serve_forever()
  def kill(self):
    if self.server:
      self.server.shutdown()
      self.join()
