# Copyright (c) 2012 Allan Wirth <allan@allanwirth.com>
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation
# files (the "Software"), to deal in the Software without
# restriction, including without limitation the rights to use,
# copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following
# conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

"""Inotify plugin for automatically adding files in dirs to the queue."""

import pyinotify
import gobject
import Queue
import os
import urllib

from ... import util
from ... import frontendplugin

class Plugin(frontendplugin.FrontendPlugin):
  __gsignals__ = {
    "file-added": (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
                   (gobject.TYPE_STRING,))
  }
  def __init__(self, player, conf):
    frontendplugin.FrontendPlugin.__init__(self, player, conf)

    self.halted = False
    self.pending = Queue.Queue()
    self.delay = self.config.getint("inotify", "delay")
    files = self.config.get("inotify", "dirs").split("|")
    for file in files:
      if file:
        self.add_watch(file)
  def run(self):
    config = self.config
    player = self.player

    class PClose(pyinotify.ProcessEvent):
      def process_IN_CLOSE_WRITE(self, event):
        f = os.path.join(event.path, event.name) if event.name else event.path
        if os.path.isfile(f):
          tmp_f = util.get_temp_filename(config,
                                         event.name if event.name else "temp")
          os.rename(f, tmp_f)
          player.add_item("file://" + urllib.quote(tmp_f), True)

    wm = pyinotify.WatchManager()
    notifier = pyinotify.Notifier(wm, PClose())
    while not self.halted:
      notifier.process_events()
      if notifier.check_events(0):
        notifier.read_events()
      try:
        path = self.pending.get(True, self.delay / 1000.0)
        wm.add_watch(path, pyinotify.EventsCodes.OP_FLAGS["IN_CLOSE_WRITE"])
      except Queue.Empty:
        pass
    notifier.stop()
  def add_watch(self, path):
    self.pending.put(path, True, None)    
  def kill(self):
    self.halted = True
    self.join()
