# Copyright (c) 2012 Allan Wirth <allan@allanwirth.com>
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation
# files (the "Software"), to deal in the Software without
# restriction, including without limitation the rights to use,
# copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following
# conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

"""Frontend for last.fm scrobbling."""

import time

from ... import frontendplugin

from . import lastfm

class Plugin(frontendplugin.FrontendPlugin):
  def __init__(self, player, conf):
    frontendplugin.FrontendPlugin.__init__(self, player, conf)
    self.to_scrobble = None
    self.player.connect("playing", self._playing)
    self.player.connect("metadata_updated", self._metadata_updated)
  def _metadata_updated(self, player, object):
    if object is self.to_scrobble:
      self.to_scrobble = object if not self._scrobble(object) else None
  def _playing(self, player, object):
    if self.to_scrobble is not None: 
      # if the metadata isn't fucking loaded by the time the next song plays
      # fuck it!
      self._scrobble(self.to_scrobble)
      self.to_scrobble = None
    if not self._scrobble(object):
      self.to_scrobble = object
  def _scrobble(self, object):
    if ("title" in object.metadata and object.metadata["title"] and
        "artist" in object.metadata and object.metadata["artist"]):
      lastfm.do_lastfm_request("track.scrobble",
                               {"timestamp[0]": str(int(time.time())),
                                "track": object.metadata["title"],
                                "artist": object.metadata["artist"],
                                "sk": self.config.get("lastfm", "sess_key")}, 
                                self.config.get("lastfm", "api_key"),
                                self.config.get("lastfm", "api_secret"),
                               True)
      return True
    return False
