# Copyright (c) 2012 Allan Wirth <allan@allanwirth.com>
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation
# files (the "Software"), to deal in the Software without
# restriction, including without limitation the rights to use,
# copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following
# conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

"""Functions for interacting with last.fm."""

import hashlib
import json
import urllib
import urllib2

BASE_URL = "http://ws.audioscrobbler.com/2.0/"
AUTH_URL = "http://www.last.fm/api/auth/"

def calc_sig(params, secret):
  keys = params.keys()
  keys.sort()
  result = ""
  for key in keys:
    result += (key + params[key])
  result += secret
  return hashlib.md5(result).hexdigest()

def do_lastfm_request(method, params, key, secret, write=False):
  params["method"] = method
  params["api_key"] = key
  sig = calc_sig(params, secret)
  params["format"] = "json"
  params["api_sig"] = sig
  data = urllib.urlencode(params)
  if not write:
    url = BASE_URL + "?" + data
    return json.load(urllib2.urlopen(url))
  else :
    return json.load(urllib2.urlopen(BASE_URL, data))
  print params
  print url

def get_auth_page(key, secret):
  token = do_lastfm_request("auth.getToken", {}, key, secret)["token"]
  params = {}
  params["token"] = token
  params["api_key"] = key
  url = AUTH_URL + "?" + urllib.urlencode(params)
  return token, url
