# Copyright (c) 2012 Allan Wirth <allan@allanwirth.com>
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation
# files (the "Software"), to deal in the Software without
# restriction, including without limitation the rights to use,
# copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following
# conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

"""An item in the playlist."""

import threading
import gobject
import os
import urllib

from . import util

class PlaylistItem(gobject.GObject):
  __gsignals__ = {
    "metadata_loaded" : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
    "stopped" : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ())
  }
  def __init__(self, id, uri, backend, temp):
    gobject.GObject.__init__(self)
    self.id = id
    self.uri = uri
    self.backend = backend
    self.temp = temp
    self.metadata = {}
    self.stopped_id = None

    self.load_metadata()
  def get_widget(self):
    return self.backend.get_widget()
  def stop(self):
    self.backend.stop()
  def is_video(self):
    return self.metadata["video"] if "video" in self.metadata else None
  def play(self):
    self.backend.play(self.uri)
    util.idle_add(self._bind_stopped)
  def _stopped(self, backend):
    self.emit("stopped")
  def _bind_stopped(self):
    self.stopped_id = self.backend.connect("stopped", self._stopped)
  def load_metadata(self):
    threading.Thread(target=self._load_metadata_thread).start()
  def _load_metadata_thread(self):
    meta = self.backend.get_metadata(self.uri)
    self.metadata.update(meta)
    util.idle_add(self.emit, "metadata_loaded")
  def unbind(self):
    if self.stopped_id is not None:
      self.backend.disconnect(self.stopped_id)
    self.cleanup()
  def cleanup(self):
    if self.temp and self.uri.startswith("file://"):
      u = urllib.unquote(self.uri[len("file://"):])
      try:
        os.unlink(u)
        os.rmdir(os.path.split(u)[0])
      except:
        pass
