# Copyright (c) 2012 Allan Wirth <allan@allanwirth.com>
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation
# files (the "Software"), to deal in the Software without
# restriction, including without limitation the rights to use,
# copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following
# conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

"""Various utility functions."""

import time
import os
import hashlib
import glib

def idle_add(func, *args, **kwargs):
  """glib.idle_add wrapper that makes wrapped function always return False,
  so that it never repeats."""
  def wrapper():
    func(*args, **kwargs)
    return False
  glib.idle_add(wrapper)

def timeout_add(interval, func, *args, **kwargs):
  def wrapper():
    func(*args, **kwargs)
    return False
  glib.timeout_add(interval, wrapper)

def format_time(time):
  time /= 1000
  return "{0}:{1:0>2}".format(time/60, time%60)

def get_resource_path(name):
  return os.path.join(os.path.dirname(__file__), "..", "res",
                           name)

def get_temp_filename(config, name):
  tmp = os.path.join(config.get("general", "cache_dir"),
                     hashlib.md5(str(time.time())).hexdigest())
  try:
    os.mkdir(tmp)
  except OSError:
    pass
  tmp_f = os.path.join(tmp, name)
  return tmp_f

def prepare_cache(cache_dir):
  try:
    os.makedirs(cache_dir)
  except OSError:
    pass

  todelete = [os.path.join(cache_dir, f) for f in os.listdir(cache_dir)]
  while len(todelete) > 0:
    f = todelete.pop(0)
    if os.path.isfile(f) and not os.path.islink(f):
      os.unlink(f)
    elif os.path.isdir(f) and not os.path.islink(f):
      try:
        os.rmdir(f)
      except OSError:
        todelete.extend([os.path.join(f, x) for x in os.listdir(f)])
        todelete.append(f)

